@@ -20,52 +20,68 @@
 
 from __future__ import division
 TEST = ('''
-Formel ein: N49° 4A.BC3 E006° 4D.EF1 und bla
-Formel ein: N49° 4A.B(A-E)2 E006° 4D.C(F-B)7 sfef
-bei N49 44,2A E6 41,2A befand
- N49 44,(B/2)(A+(3*7)3) E6 41,B(A-9) sta
-ei
-N 49° 44.(TRRRS) - (S*S)
-E 006° 41.RSR
-PS
-n bei: N 49° 45.999-12*A-62 E 006° 38.999-12*A-60', {}),
-bei N 49° 45.EBA E 006° 37.DFC und', {}),
- zu N 49° 45.(A B C)+7 E 006° 38.(D E F)- 138
-bei
-ei N 49° 45.(A-1)(C-1)(B*E+5) E 006° 37.(B+1)(A)(E+1). ... bei N 49° 45.(G/3)(H)(H-D) E 006° 36. (H-2)(A-D-F)(G/3). Am Referenzpunkt N 49° 45.((G+1)/(A*2))(A)((R-S+1)*D) E 006° 36. (D+1)(H)(1)
+
+
+
 ''', {'A': 2, 'D': 4, 'G': 3,'T': 1, 'R': 2, 'S': 1, 'H': 4, 'B': 2, 'C': 9, 'E': 0, 'F': 1})
 HTML = '''
 
-			        <td>Note:</td> 
-			        <td colspan="4">Hier findest du die erste Dose mit weiteren Informationen. S3 findest du bei<p>N49° 21.10(A-1)' E6° 40.(A-1)04'</td> 
-			        <td>&nbsp;</td> 
-		        </tr>  
+<br /></p> 
+<p><font face="Arial, sans-serif"><font size=
+"4"><b>Final:</b></font></font></p> 
+<p style="font-style: normal"><font color="#000000"><font face=
+"Arial, sans-serif"><font size="3"><b><span style=
+"background: transparent">N 49°
+(B-C+A+0,5*D).(F+D)(F-G)(C-2*A)</span></b></font></font></font></p> 
+<p style="font-style: normal"><font color="#000000"><font face=
+"Arial, sans-serif"><font size="3"><b><span style=
+"background: transparent">E 6°
+(2*A+C).(G-E)(B-C+0,5*D)(F-D)</span></b></font></font></font></p> 
+<p style="font-style: normal; font-weight: normal"><font color=
+"#000000"><font face="Arial, sans-serif"><font size=
+"3"><span style="background: transparent">Es müssen keine Zäune
+oder Mauern überwunden werden.</span></font></font></font></p>
+N 49°
+(B-C+A+0,5*D).(F+D)(F-G)(D-2*B)
+<p><br /> 
+<br /></p></span> 
+                
+            </div> 
+            <p> 
+</p> 
 '''
 import geo
 import re
+import logging
+logger = logging.getLogger('coordfinder')
 
-class CalcCoordinateManager():
-    def __init__(self, cache, text):
-        self.vars = cache.get_vars()
+class CalcCoordinateManager(object):
+    def __init__(self, cache, text, vars):
+        self.__vars = vars
         self.coords, self.requires = CalcCoordinate.find(text)
         self.update()
         
     def set_var(self, char, value):
         if value != '':
-            self.vars[char] = value
+            self.__vars[char] = value
         else:
-            del self.vars[char]
+            del self.__vars[char]
         self.update()
 
     def update(self):
         for c in self.coords:
-            c.set_vars(self.vars)
+            c.set_vars(self.__vars)
             if c.has_requires():
                 result = c.try_get_solution()
 
     def get_solutions(self):
-        return [c.result for c in self.coords if c.has_requires()]
-    
+        return [c.result for c in self.coords if c.has_requires() and len(c.requires) > 0]
+
+    def get_plain_coordinates(self):
+        return [c.result for c in self.coords if len(c.requires) == 0]
+
+    def get_vars(self):
+        return self.__vars
         
 
 class CalcCoordinate():
@@ -96,7 +112,7 @@ class CalcCoordinate():
         self.vars = {}
 
     def prepare(self, text):
-        return re.sub('[^A-Za-z()+*/0-9-]', '', text)
+        return (re.sub('[^A-Za-z()+*/0-9-.,]', '', text)).replace(',', '.')
 
     def set_vars(self, var):
         self.warnings = []
@@ -104,7 +120,7 @@ class CalcCoordinate():
 
     def has_requires(self):
         for i in self.requires:
-            if not i in self.vars.keys():
+            if not i in self.vars:
                 return False
         return True
 
@@ -165,36 +181,41 @@ class CalcCoordinate():
 
     @staticmethod
     def find(text):
+        #logger.debug('Processing Text: \n ------------------------ \n%s\n ----------------------' % text)
         foundsigs = []
-        text = re.sub(ur'''\s[a-zA-Z]{2,}\s''', ' | ', text)
-        text = re.sub(ur'''\b[a-zA-Z]{4,}\b''', ' | ', text)
+        text = re.sub(ur'''(?u)\s[^\W\d_]{2,}\s''', ' | ', text)
+        text = re.sub(ur'''(?u)\b[^\W\d_]{4,}\b''', ' | ', text)
         text = text.replace('°', '|')
         text = text.replace(unichr(160), ' ')
         text = re.sub(ur''' +''', ' ', text)
+        #logger.debug('Processed Text: \n ------------------------ \n%s\n ----------------------' % text)
 
-        matches = re.findall(ur'''(?<![a-zA-Z])([NSns])\s?([A-Z() -+*/0-9]+?)[\s|]{1,2}([A-Za-z ()+*/0-9-]+)[.,\s]([A-Za-z ()+*/0-9-]+)['`\s,/]+([EOWeow])\s?([A-Za-z() +*/0-9-]+?)[\s|]{1,2}([A-Za-z ()+*/0-9-]+)[.,\s]([A-Za-z ()+*/0-9-]+)[\s'`]*(?![a-zA-Z])''', text)
+        single_calc_part = ur'''((?:\([A-Za-z +*/0-9-.,]+\)|[A-Za-z ()+*/0-9-])+)'''
+        matches = re.findall(ur'''(?<![a-zA-Z])([NSns])\s?([A-Z() -+*/0-9]+?)[\s|]{1,2}%(calc)s[.,\s]%(calc)s['`\s,/]+([EOWeow])\s?([A-Z() -+*/0-9]+?)[\s|]{1,2}%(calc)s[.,\s]%(calc)s[\s'`]*(?![a-zA-Z])''' % {'calc' : single_calc_part}, text)
         found = []
         requires = set()
         for match in matches:
-            sig = "|".join(re.sub('[^A-Za-z()+*/0-9-]+', '', x) for x in match)
+            sig = "|".join(re.sub('[^A-Za-z()+*/0-9-.,]+', '', x) for x in match)
 
             if sig in foundsigs:
                 continue
             foundsigs.append(sig)
+            logger.info("Found match: " + repr(match))
             c = CalcCoordinate(*match)
-            if len(c.requires) == 0:
-                continue
+            #if len(c.requires) == 0:
+            #    continue
             found.append(c)
             requires |= c.requires
         return (found, requires)
-'''
+
 if __name__ == "__main__":
     from simplegui import SimpleGui
     print '\n\n========================================================='
     h = SimpleGui._strip_html(HTML) 
     print h
-    for x in h:
-        print "%d -> %s" % (ord(x), x)
+    #for x in h:
+    #    print "%d -> %s" % (ord(x), x)
     print '---------------------------------------------------------'
-    CalcCoordinate.find(h)
-'''
+    for instance in CalcCoordinate.find(h)[0]:
+        print "Found: %s" % (instance.orig)
+
