@@ -8,34 +8,41 @@ from copy import copy
 class PointProvider():
     MAX_RESULTS = 1000
 
-    def __init__(self, filename, downloader, ctype, table):
+    def __init__(self, filename, ctype, table):
 
         self.filterstack = []
         self.conn = connect(filename)
         self.conn.row_factory = Row
         self.conn.text_factory = str
         self.ctype = ctype
-        self.downloader = downloader
         self.cache_table = table
         self.filterstring = []
         self.filterargs = []
-        c = self.conn.cursor()
-        c.execute('PRAGMA temp_store = MEMORY;')
-        # yes, this setting is a bit dangerous for the database, but the 
-        # advantages outbalance unlikely database corruption
-        c.execute('PRAGMA synchronous=OFF')
-        c.execute('CREATE TABLE IF NOT EXISTS %s (%s)' % (self.cache_table, ', '.join([' '.join(m) for m in self.ctype.SQLROW.items()])))
+
+        # yes, the synchronous=off setting is a bit dangerous for the database,
+        # but the advantages outbalance unlikely database corruption
+        self.conn.executescript(
+            'PRAGMA temp_store = MEMORY;' \
+            'PRAGMA synchronous=OFF;' \
+            'CREATE TABLE IF NOT EXISTS %s (%s);' % (self.cache_table, ', '.join('%s %s' % m for m in self.ctype.SQLROW.items())))
         self.check_table()
-        c.execute('CREATE INDEX IF NOT EXISTS %s_latlon ON %s (lat ASC, lon ASC)' % (self.cache_table, self.cache_table))
-        c.execute('CREATE INDEX IF NOT EXISTS %s_name ON %s (name ASC)' % (self.cache_table, self.cache_table))
-        c.close()
+        self.conn.executescript(
+            'CREATE INDEX IF NOT EXISTS %(table)s_latlon ON %(table)s (lat ASC, lon ASC);' \
+            'CREATE INDEX IF NOT EXISTS %(table)s_name ON %(table)s (name ASC);' \
+            'CREATE INDEX IF NOT EXISTS %(table)s_fieldnote ON %(table)s (logas);' % {'table' : self.cache_table}
+            )
+
+
+        #c.execute('CREATE TABLE IF NOT EXISTS %s (%s)' % (self.cache_table, ', '.join(' '.join(m) for m in self.ctype.SQLROW.items())))
+        #self.check_table()
+        #c.execute('CREATE INDEX IF NOT EXISTS %s_latlon ON %s (lat ASC, lon ASC)' % (self.cache_table, self.cache_table))
 
     def check_table(self):
         c = self.conn.cursor()
         fields = copy(self.ctype.SQLROW)
         c.execute('PRAGMA TABLE_INFO(%s)' % self.cache_table)
         for row in c:
-            if row[1] in fields.keys():
+            if row[1] in fields:
                 del fields[row[1]]
 
         # add all remaining fields
@@ -60,7 +67,7 @@ class PointProvider():
     def add_point(self, p, replace=False):
                 
         if replace:
-            self.conn.execute("INSERT OR REPLACE INTO %s (`%s`) VALUES (%s)" % (self.cache_table, '`, `'.join(self.ctype.SQLROW.keys()), ', '.join([':%s' % k for k in self.ctype.SQLROW.keys()])), p.serialize())
+            self.conn.execute("INSERT OR REPLACE INTO %s (`%s`) VALUES (%s)" % (self.cache_table, '`, `'.join(self.ctype.SQLROW.keys()), ', '.join(':%s' % k for k in self.ctype.SQLROW.keys())), p.serialize())
             return None
         else:
             c = self.conn.cursor()
@@ -72,7 +79,7 @@ class PointProvider():
                 self.conn.execute("UPDATE %s SET found = ?, type = ?, lat = ?, lon = ?, status = ? WHERE name = ?" % self.cache_table, (p.found, p.type, p.lat, p.lon, p.status, p.name))
                 return False
             else:
-                self.conn.execute("INSERT INTO %s (`%s`) VALUES (%s)" % (self.cache_table, '`, `'.join(self.ctype.SQLROW.keys()), ', '.join([':%s' % k for k in self.ctype.SQLROW.keys()])), p.serialize())
+                self.conn.execute("INSERT INTO %s (`%s`) VALUES (%s)" % (self.cache_table, '`, `'.join(self.ctype.SQLROW.keys()), ', '.join(':%s' % k for k in self.ctype.SQLROW.keys())), p.serialize())
                 return True
             
                 
@@ -87,21 +94,17 @@ class PointProvider():
         
     # should never ever be used with anything except a user provided query
     def get_by_query(self, query):
-        c = self.conn.cursor()
 
-        c.execute(query)
+        c = self.conn.execute(query)
         return self.pack_result(c)
                 
     def get_points(self, c1, c2):
                 
-        c = self.conn.cursor()
-
-        c.execute('SELECT * FROM %s WHERE (lat BETWEEN ? AND ?) AND (lon BETWEEN ? AND ?)' % self.cache_table, (min(c1.lat, c2.lat), max(c1.lat, c2.lat), min(c1.lon, c2.lon), max(c1.lon, c2.lon)))
+        c = self.conn.execute('SELECT * FROM %s WHERE (lat BETWEEN ? AND ?) AND (lon BETWEEN ? AND ?)' % self.cache_table, (min(c1.lat, c2.lat), max(c1.lat, c2.lat), min(c1.lon, c2.lon), max(c1.lon, c2.lon)))
         return self.pack_result(c)
                 
     def get_titles_and_names(self):
-        c = self.conn.cursor()
-        c.execute('SELECT name, title FROM %s' % self.cache_table)
+        c = self.conn.execute('SELECT name, title FROM %s' % self.cache_table)
         strings = []
         for row in c:
             strings.append(row['name'])
@@ -110,17 +113,13 @@ class PointProvider():
         return strings
             
     def get_new_fieldnotes_count(self):
-        c = self.conn.cursor()
-
-        c.execute('SELECT count(*) AS cnt FROM %s WHERE logas != %d' % (self.cache_table, self.ctype.LOG_NO_LOG))
+        c = self.conn.execute('SELECT count(*) AS cnt FROM %s WHERE logas != %d' % (self.cache_table, self.ctype.LOG_NO_LOG))
         for row in c:
             return row['cnt']
         return 0
 
     def get_new_fieldnotes(self):
-        c = self.conn.cursor()
-
-        c.execute('SELECT * FROM %s WHERE logas != %d' % (self.cache_table, self.ctype.LOG_NO_LOG))
+        c = self.conn.execute('SELECT * FROM %s WHERE logas != %d' % (self.cache_table, self.ctype.LOG_NO_LOG))
         return self.pack_result(c)
 
         
@@ -139,12 +138,12 @@ class PointProvider():
         elif found == False:
             filterstring.append('(found = 0)')
     
-        c = self.conn.cursor()
+        
         # we don't have 'power' or other advanced mathematic operators
         # in sqlite, so doing distance calculation in python
         query = 'SELECT * FROM %s WHERE %s' % (self.cache_table, " AND ".join(filterstring))
                 
-        c.execute(query, tuple(filterargs))
+        c = self.conn.execute(query, tuple(filterargs))
 
         mindist = () # we use this as positive infinity
         mindistrow = None
@@ -158,8 +157,7 @@ class PointProvider():
                 mindist = dist
         if mindistrow == None:
             return None
-        coord = self.ctype(mindistrow['lat'], mindistrow['lon'])
-        coord.unserialize(mindistrow)
+        coord = self.ctype(mindistrow['lat'], mindistrow['lon'], '', mindistrow)
         return coord
                 
     def set_filter(self, found=None, has_details=None, owner_search='', name_search='', size=None, terrain=None, diff=None, ctype=None, adapt_filter=False, marked=None):
@@ -198,7 +196,7 @@ class PointProvider():
             filterstring.append("((name LIKE '%%%s%%') OR (title LIKE '%%%s%%'))" % (name_search, name_search))
                         
         if size != None:
-            filterstring.append('(size IN (%s))' % (", ".join([str(b) for b in size])))
+            filterstring.append('(size IN (%s))' % (", ".join(str(b) for b in size)))
 
         if terrain != None:
             if type(terrain) == tuple:
@@ -206,7 +204,7 @@ class PointProvider():
                 filterargs.append(terrain[0] * 10)
                 filterargs.append(terrain[1] * 10)
             elif type(terrain) == list:
-                filterstring.append('(terrain IN (%s))' % (", ".join(['?' for b in terrain])))
+                filterstring.append('(terrain IN (%s))' % (", ".join('?' for b in terrain)))
                 for b in terrain:
                     filterargs.append(b * 10)
 
@@ -217,13 +215,13 @@ class PointProvider():
                 filterargs.append(diff[0] * 10)
                 filterargs.append(diff[1] * 10)
             elif type(diff) == list:
-                filterstring.append('(difficulty IN (%s))' % (", ".join(['?' for b in diff])))
+                filterstring.append('(difficulty IN (%s))' % (", ".join('?' for b in diff)))
                 for b in diff:
                     filterargs.append(b * 10)
                         
         if ctype != None:
             if len(ctype) > 0:
-                filterstring.append('(type IN (%s))' % (", ".join(['?' for b in ctype])))
+                filterstring.append('(type IN (%s))' % (", ".join('?' for b in ctype)))
                 for b in ctype:
                     filterargs.append(b)
                                         
@@ -260,36 +258,50 @@ class PointProvider():
         elif found == False:
             filterstring.append('(found = 0)')
 
-        c = self.conn.cursor()
+        
         query = 'SELECT * FROM %s WHERE %s LIMIT %s' % (self.cache_table, " AND ".join(filterstring), max_results)
 
-        c.execute(query, tuple(filterargs))
+        c = self.conn.execute(query, tuple(filterargs))
         return self.pack_result(c)
 
     def pack_result(self, cursor):
-        points = []
-        for row in cursor:
-            coord = self.ctype(row['lat'], row['lon'])
-            coord.unserialize(row)
-            points.append(coord)
+        points = [self.ctype(None, None, None, row) for row in cursor]
         cursor.close()
         return points
                 
     def find_by_string(self, string):
         query = 'SELECT * FROM %s WHERE name LIKE ? OR title LIKE ? LIMIT 2' % self.cache_table
-        c = self.conn.cursor()
-        c.execute(query, (string, string))
+        c = self.conn.execute(query, (string, string))
         row = c.fetchone()
-        coord = self.ctype(row['lat'], row['lon'])
-        coord.unserialize(row)
+        coord = self.ctype(None, None, None, row)
                 
         # we cannot reliably determine # of results, so using workaround here
         if c.fetchone() != None:
             return None
         return coord
                 
-    def update_field(self, coordinate, field, newvalue):
+    def update_field(self, coordinate, field, newvalue, save = True):
         query = 'UPDATE %s SET %s = ? WHERE name = ?' % (self.cache_table, field)
-        c = self.conn.cursor()
-        c.execute(query, (newvalue, coordinate.name))
+        self.conn.execute(query, (newvalue, coordinate.name))
+        if save:
+            self.save()
+
+    def get_by_name(self, gcname):
+        query = 'SELECT * FROM %s WHERE name LIKE ? LIMIT 2' % self.cache_table
+        c = self.conn.execute(query, (gcname,))
+        row = c.fetchone()
+        if row != None:
+            coord = self.ctype(None, None, None, row)
+            return coord
+        else:
+            return None
+
+    def remove_geocaches(self, list):
+        names = [x.name for x in list if x.name != '']
+        query = 'DELETE FROM %s WHERE name IN (%s)' % (self.cache_table, (','.join('?' for x in names)))
+        
+        self.conn.execute(query, tuple(names))
         self.save()
+
+    def optimize(self):
+        self.conn.execute('VACUUM')
